// Copyright Epic Games, Inc. All Rights Reserved.

#include "PeelVCamStyle.h"
#include "PeelVCam.h"
#include "Framework/Application/SlateApplication.h"
#include "Styling/SlateStyleRegistry.h"
#include "Slate/SlateGameResources.h"
#include "Interfaces/IPluginManager.h"
#include "Styling/SlateStyleMacros.h"

#define RootToContentDir Style->RootToContentDir

TSharedPtr<FSlateStyleSet> FPeelVCamStyle::StyleInstance = nullptr;

void FPeelVCamStyle::Initialize()
{
	if (!StyleInstance.IsValid())
	{
		StyleInstance = Create();
		FSlateStyleRegistry::RegisterSlateStyle(*StyleInstance);
	}
}

void FPeelVCamStyle::Shutdown()
{
	FSlateStyleRegistry::UnRegisterSlateStyle(*StyleInstance);
	ensure(StyleInstance.IsUnique()); 
	StyleInstance.Reset();
}

FName FPeelVCamStyle::GetStyleSetName()
{
	static FName StyleSetName(TEXT("PeelVCamStyle"));
	return StyleSetName;
}


const FVector2D Icon16x16(16.0f, 16.0f);
const FVector2D Icon20x20(20.0f, 20.0f);

TSharedRef< FSlateStyleSet > FPeelVCamStyle::Create()
{
	TSharedRef< FSlateStyleSet > Style = MakeShareable(new FSlateStyleSet("PeelVCamStyle"));
	Style->SetContentRoot(IPluginManager::Get().FindPlugin("PeelVCam")->GetBaseDir() / TEXT("Resources"));

	Style->Set("PeelVCam.PluginAction", new IMAGE_BRUSH_SVG(TEXT("Icon128"), Icon20x20));
	return Style;
}

void FPeelVCamStyle::ReloadTextures()
{
	if (FSlateApplication::IsInitialized())
	{
		FSlateApplication::Get().GetRenderer()->ReloadTextureResources();
	}
}

const ISlateStyle& FPeelVCamStyle::Get()
{
	return *StyleInstance;
}
