#include "PeelVCam.h"
#include "Interfaces/IPluginManager.h"
#include "PeelVCamStyle.h"
#include "PeelVCamCommands.h"
#include "Misc/MessageDialog.h"
#include "Json.h"
#include "PeelWireless.h"
#include "ToolMenus.h"
#include "EditorUtilitySubsystem.h"
#include "EditorUtilityWidgetBlueprint.h"
#include "EditorAssetLibrary.h"
#include "WidgetBlueprint.h"

static const FName PeelVCamTabName("PeelVCam");

#define LOCTEXT_NAMESPACE "FPeelVCamModule"

void FPeelVCamModule::StartupModule()
{

	FPeelVCamStyle::Initialize();
	FPeelVCamStyle::ReloadTextures();

	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module

	IPluginManager& PluginManager = IPluginManager::Get();

	FPluginDescriptor PluginDescriptor = PluginManager.FindPlugin(TEXT("PeelVCam"))->GetDescriptor();

	auto json = PluginDescriptor.CachedJson;

	//TSharedPtr<FJsonObject, ESPMode::ThreadSafe> json =

	int port = json->GetIntegerField("VCamPort");
	if (port == 0) { port = 9158; }

	PeelWireless = new UPeelWirelessThread();
	PeelWireless->Start(port);

	ISequencerModule& SequencerModule = FModuleManager::Get().LoadModuleChecked<ISequencerModule>("Sequencer");
	OnSequencerCreatedHandle = SequencerModule.RegisterOnSequencerCreated(FOnSequencerCreated::FDelegate::CreateRaw(this, &FPeelVCamModule::OnSequencerCreated));

	FPeelVCamCommands::Register();

	PluginCommands = MakeShareable(new FUICommandList);

	PluginCommands->MapAction(
		FPeelVCamCommands::Get().PluginAction,
		FExecuteAction::CreateRaw(this, &FPeelVCamModule::PluginButtonClicked),
		FCanExecuteAction());

	UToolMenus::RegisterStartupCallback(FSimpleMulticastDelegate::FDelegate::CreateRaw(this, &FPeelVCamModule::RegisterMenus));
}

void FPeelVCamModule::ShutdownModule()
{
	UToolMenus::UnRegisterStartupCallback(this);

	UToolMenus::UnregisterOwner(this);

	FPeelVCamStyle::Shutdown();

	FPeelVCamCommands::Unregister();

	PeelWireless->Stop();

	ISequencerModule* SequencerModule = FModuleManager::GetModulePtr<ISequencerModule>("Sequencer");
	if (SequencerModule)
	{
		SequencerModule->UnregisterOnSequencerCreated(OnSequencerCreatedHandle);
	}
}

void FPeelVCamModule::PluginButtonClicked()
{
	UObject* Blueprint = UEditorAssetLibrary::LoadAsset(FString(TEXT("EditorUtilityWidgetBlueprint'/PeelVCam/PeelVCamTool.PeelVCamTool'")));
	if (IsValid(Blueprint))
	{
		UEditorUtilityWidgetBlueprint* EditorWidget = Cast<UEditorUtilityWidgetBlueprint>(Blueprint);
		if (IsValid(EditorWidget))
		{
			UEditorUtilitySubsystem* EditorUtilitySubsystem = GEditor->GetEditorSubsystem<UEditorUtilitySubsystem>();
			EditorUtilitySubsystem->SpawnAndRegisterTab(EditorWidget);
		}
	}
}



void FPeelVCamModule::RegisterMenus()
{
	// Owner will be used for cleanup in call to UToolMenus::UnregisterOwner
	FToolMenuOwnerScoped OwnerScoped(this);

	{
		UToolMenu* Menu = UToolMenus::Get()->ExtendMenu("LevelEditor.MainMenu.Window");
		{
			FToolMenuSection& Section = Menu->FindOrAddSection("WindowLayout");
			Section.AddMenuEntryWithCommandList(FPeelVCamCommands::Get().PluginAction, PluginCommands);
		}
	}

	{
		UToolMenu* ToolbarMenu = UToolMenus::Get()->ExtendMenu("LevelEditor.LevelEditorToolBar.PlayToolBar");
		{
			FToolMenuSection& Section = ToolbarMenu->FindOrAddSection("PluginTools");
			{
				FToolMenuEntry& Entry = Section.AddEntry(FToolMenuEntry::InitToolBarButton(FPeelVCamCommands::Get().PluginAction));
				Entry.SetCommandList(PluginCommands);
			}
		}
	}
}

void FPeelVCamModule::Register(UPeelWireless *ref)
{
	PeelWireless->refs.Add(ref);
}

void FPeelVCamModule::Remove(UPeelWireless* ref)
{
	PeelWireless->refs.Remove(ref);
}

#pragma optimize("", off)
void FPeelVCamModule::OnSequencerCreated(TSharedRef < ISequencer> Sequencer)
{
	//SequencerRef = Sequencer;
	test++;

	Sequencer->OnRecordEvent().AddLambda([this]
		{
			test++;
		});

	Sequencer->OnStopEvent().AddLambda([this]
		{
			test++;
		});


	Sequencer->OnPlayEvent().AddLambda([this]
		{
			test++;
		});


}
#pragma optimize("", on)

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FPeelVCamModule, PeelVCam)