#include "PeelWirelessThread.h"
#include "PeelWireless.h"
#include "Common/UdpSocketBuilder.h"
#include "IPAddress.h"
#include "SocketSubsystem.h"
#include <string>
#include "GenericPlatform/GenericPlatformProcess.h"
#include "LevelSequenceEditorBlueprintLibrary.h" 
#include "ExtensionLibraries/MovieSceneSequenceExtensions.h"

#if ENGINE_MAJOR_VERSION >= 5 && ENGINE_MINOR_VERSION >= 3
	#include "FunctionLibraries/VCamBlueprintFunctionLibrary.h"
#else 
	#include "VCamBlueprintFunctionLibrary.h"
#endif

PEELVCAM_API UPeelWirelessThread::UPeelWirelessThread()
	: Thread(nullptr)
	, bRunThread(false)
	, mSocket(nullptr)
	, mPort(-1)
	, mLens(50.0f)
{
}

PEELVCAM_API UPeelWirelessThread::~UPeelWirelessThread()
{
	if (Thread)
	{
		bRunThread = false;
		Thread->Kill();
		delete Thread;
	}
}

PEELVCAM_API void  UPeelWirelessThread::Start(int p)
{
	mPort = p;
	Thread = FRunnableThread::Create(this, TEXT("Peel Com Thread"));
}

PEELVCAM_API void UPeelWirelessThread::Stop()
{
	bRunThread = false;
}

PEELVCAM_API bool UPeelWirelessThread::Init()
{
	return true;
}


#pragma optimize("", off)
PEELVCAM_API uint32 UPeelWirelessThread::Run()
{
	bRunThread = true;

	mSocket = FUdpSocketBuilder("builder")
		.AsBlocking()
		.BoundToPort(mPort)
		.Build();

	if (!mSocket)
		return 1;

	int32 BytesRead;
	TSharedRef<FInternetAddr> targetAddr = ISocketSubsystem::Get(PLATFORM_SOCKETSUBSYSTEM)->CreateInternetAddr();

	std::string line;

	bool updateOffsetTranslation;
	bool updateOffsetRotation;
	bool updateDampingTranslation;
	bool updateDampingRotation;
	bool updateCommand;
	TArray<FText> commands;

	float lens, focusDistance, aperture, rawVal, frameVal;
	float sensorWidth, sensorHeight, pixelAspect;
	char sensorName[100];
	int buttonNum;
	int valueNum;
	int landmarkID;
	bool debugFocus = false;

	FVector offsetTranslation;
	FVector offsetRotation;
	FVector dampingTranslation;
	float dampingRotation;

	while (bRunThread)
	{
		char buf[1024];
		bool ok = true;
		while (bRunThread)
		{
			updateOffsetTranslation = false;
			updateOffsetRotation = false;
			updateDampingTranslation = false;
			updateDampingRotation = false;
			updateCommand = false;

			if (mSocket->RecvFrom((uint8*)buf, 1024, BytesRead, *targetAddr))
			{	
				/* LANDMARKS */
				if (strncmp(buf, "addlandmark", 11) == 0) {
					landmarkID = atoi(buf + 11);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, landmarkID]() { ref->AddLandmark(landmarkID); });
					}
					continue;
				}
				if (strncmp(buf, "landmark", 8) == 0) {
					landmarkID = atoi(buf + 8);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, landmarkID]() { ref->GotoLandmark(landmarkID); });
					}
					continue;
				}
				/* ZOOM */
				if (strncmp(buf, "LENS=", 5) == 0) {
					lens = atof(buf + 5);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, lens]() { ref->SetLens(lens); });
					}
					continue;
				}
				/* FILMBACK */
				if (strncmp(buf, "SENSOR:", 7) == 0) {
					// Parse the line
					sscanf(buf, "SENSOR: \"%[^\"]\" %f %f %f", sensorName, &sensorWidth, &sensorHeight, &pixelAspect);

					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, sensorWidth, sensorHeight, pixelAspect]() { ref->SetFilmback(sensorWidth, sensorHeight, pixelAspect); });
					}
					continue;
				}
				/* APERTURE */
				if (strncmp(buf, "APERTURE=", 9) == 0) {
					aperture = atof(buf + 9);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, aperture]() { ref->SetAperture(aperture); });
					}
					continue;
				}
				/* FOCUS */
				if (strncmp(buf, "FOCUS=", 6) == 0) {
					focusDistance = atof(buf + 6);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, focusDistance]() { ref->SetFocusDistance(focusDistance); });
					}
					continue;
				}
				/* SEQUENCER */
				if (strncmp(buf, "FRAME=", 6) == 0) {
					frameVal = atof(buf + 6);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, frameVal]() { ref->SetFrame(frameVal); });
					}
					continue;
				}
				if (strncmp(buf, "decrementframe", 15) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->DecrementFrame(); });
					}
					continue;
				}
				if (strncmp(buf, "incrementframe", 15) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->IncrementFrame(); });
					}
					continue;
				}
				if (strncmp(buf, "beginningframe", 15) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->BeginningFrame(); });
					}
					continue;
				}
				if (strncmp(buf, "playtoggle", 11) == 0) {
					AsyncTask(ENamedThreads::GameThread, [&]() {
						if (UVCamBlueprintFunctionLibrary::IsCurrentLevelSequencePlaying()) {
							UVCamBlueprintFunctionLibrary::PauseCurrentLevelSequence();
						} else {
							UVCamBlueprintFunctionLibrary::PlayCurrentLevelSequence();
						}
					});
					continue;
				}
				if (strncmp(buf, "play", 5) == 0) {
					AsyncTask(ENamedThreads::GameThread, [&]() { UVCamBlueprintFunctionLibrary::PlayCurrentLevelSequence(); });
					continue;
				}
				if (strncmp(buf, "playstop", 9) == 0) {
					AsyncTask(ENamedThreads::GameThread, [&]() { UVCamBlueprintFunctionLibrary::PauseCurrentLevelSequence(); });
					continue;
				}
				if (strncmp(buf, "adddollykey", 12) == 0) {
					// Create a new array to store non-TRASH instances
					TArray<UPeelWireless*> nonTrashRefs;

					// Iterate through each element in refs
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						if (ref->GetName().Contains("TRASH")) {
							continue; // Skip elements containing "TRASH"
						}
						// Add non-TRASH elements to the new array
						nonTrashRefs.Add(ref);
					}
					// Remove duplicate elements based on memory address
					for (int32 Index = nonTrashRefs.Num() - 1; Index >= 0; --Index) {
						UPeelWireless* CurrentRef = nonTrashRefs[Index];
						int32 NumFound = 0;
						for (int32 ComparisonIndex = nonTrashRefs.Num() - 1; ComparisonIndex >= 0; --ComparisonIndex) {
							if (nonTrashRefs[ComparisonIndex] == CurrentRef) {
								NumFound++;
								if (NumFound > 1) {
									nonTrashRefs.RemoveAt(ComparisonIndex);
								}
							}
						}
					}

					// Perform the AddDollyTransformKey operation on each remaining element
					for (int i = 0; i < nonTrashRefs.Num(); i++) {
						UPeelWireless* ref = nonTrashRefs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->AddDollyTransformKey(); });
					}
					continue;
				}
				if (strncmp(buf, "cleardollykeys", 15) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->ClearDollyKeys(); });
					}
					continue;
				}
				if (strncmp(buf, "prevdollykey", 13) == 0) {
					// Create a new array to store non-TRASH instances
					TArray<UPeelWireless*> nonTrashRefs;

					// Iterate through each element in refs
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						if (ref->GetName().Contains("TRASH")) {
							continue; // Skip elements containing "TRASH"
						}
						// Add non-TRASH elements to the new array
						nonTrashRefs.Add(ref);
					}
					// Remove duplicate elements based on memory address
					for (int32 Index = nonTrashRefs.Num() - 1; Index >= 0; --Index) {
						UPeelWireless* CurrentRef = nonTrashRefs[Index];
						int32 NumFound = 0;
						for (int32 ComparisonIndex = nonTrashRefs.Num() - 1; ComparisonIndex >= 0; --ComparisonIndex) {
							if (nonTrashRefs[ComparisonIndex] == CurrentRef) {
								NumFound++;
								if (NumFound > 1) {
									nonTrashRefs.RemoveAt(ComparisonIndex);
								}
							}
						}
					}

					// Perform the AddDollyTransformKey operation on each remaining element
					for (int i = 0; i < nonTrashRefs.Num(); i++) {
						UPeelWireless* ref = nonTrashRefs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->PrevDollyKey(); });
					}
					continue;
				}
				if (strncmp(buf, "nextdollykey", 13) == 0) {
					// Create a new array to store non-TRASH instances
					TArray<UPeelWireless*> nonTrashRefs;

					// Iterate through each element in refs
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						if (ref->GetName().Contains("TRASH")) {
							continue; // Skip elements containing "TRASH"
						}
						// Add non-TRASH elements to the new array
						nonTrashRefs.Add(ref);
					}
					// Remove duplicate elements based on memory address
					for (int32 Index = nonTrashRefs.Num() - 1; Index >= 0; --Index) {
						UPeelWireless* CurrentRef = nonTrashRefs[Index];
						int32 NumFound = 0;
						for (int32 ComparisonIndex = nonTrashRefs.Num() - 1; ComparisonIndex >= 0; --ComparisonIndex) {
							if (nonTrashRefs[ComparisonIndex] == CurrentRef) {
								NumFound++;
								if (NumFound > 1) {
									nonTrashRefs.RemoveAt(ComparisonIndex);
								}
							}
						}
					}

					// Perform the AddDollyTransformKey operation on each remaining element
					for (int i = 0; i < nonTrashRefs.Num(); i++) {
						UPeelWireless* ref = nonTrashRefs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->NextDollyKey(); });
					}
					continue;
				}
				if (strncmp(buf, "debugfocus", 11) == 0) {
					debugFocus = !debugFocus;
					for (int i = 0; i  < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, debugFocus]() { ref->SetDebugFocus(debugFocus); });
					}
					continue;
				}

				/* NAV */
				if (strncmp(buf, "180", 3) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->TurnCameraBy(180); });
					}
					continue;
				}
				if (strncmp(buf, "-90", 3) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->TurnCameraBy(-90); });
					}
					continue;
				}
				if (strncmp(buf, "+90", 2) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->TurnCameraBy(90); });
					}
					continue;
				}
				if (strncmp(buf, "ZEROMC", 7) == 0) {
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref]() { ref->ZeroMagicCarpet(); });
					}
					continue;
				}
				if (strncmp(buf, "X=", 2) == 0) {
					offsetTranslation.X = atof(buf + 2);
					updateOffsetTranslation = true;
				}

				if (strncmp(buf, "Y=", 2) == 0) {
					offsetTranslation.Y = atof(buf + 2);
					updateOffsetTranslation = true;
				}

				if (strncmp(buf, "Z=", 2) == 0) {
					offsetTranslation.Z = atof(buf + 2);
					updateOffsetTranslation = true;
				}
				
				if (strncmp(buf, "RX=", 3) == 0) {
					offsetRotation.X = atof(buf + 3);
					updateOffsetRotation = true;
				}

				if (strncmp(buf, "RY=", 3) == 0) {
					offsetRotation.Y = atof(buf + 3);
					updateOffsetRotation = true;
				}

				if (strncmp(buf, "RZ=", 3) == 0) {
					offsetRotation.Z = atof(buf + 3);
					updateOffsetRotation = true;
				}
				if (strncmp(buf, "DAMP-TX=", 8) == 0) {
					dampingTranslation.X = atof(buf + 8);
					updateDampingTranslation = true;
				}
				if (strncmp(buf, "DAMP-TY=", 8) == 0) {
					dampingTranslation.Y = atof(buf + 8);
					updateDampingTranslation = true;
				}
				if (strncmp(buf, "DAMP-TZ=", 8) == 0) {
					dampingTranslation.Z = atof(buf + 8);
					updateDampingTranslation = true;
				}
				if (strncmp(buf, "DAMP-ROT=", 9) == 0) {
					dampingRotation = atof(buf + 9);
					updateDampingRotation = true;
				}
				if (updateOffsetTranslation)
				{
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, offsetTranslation]() { ref->SetOffsetTranslation(offsetTranslation); });
						updateOffsetTranslation = false;
					}
					continue;
				}
				if (updateOffsetRotation)
				{
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, offsetRotation]() { ref->SetOffsetRotation(offsetRotation); });
						updateOffsetRotation = false;
					}
					continue;
				}
				if (updateDampingTranslation)
				{
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, dampingTranslation]() { ref->SetDampingTranslation(dampingTranslation); });
						updateDampingTranslation = false;
					}
					continue;
				}
				if (updateDampingRotation)
				{
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, dampingRotation]() { ref->SetDampingRotation(dampingRotation); });
						updateDampingRotation = false;
					}
					continue;
				}
				/* MISC */
				if (strncmp(buf, "button", 6) == 0) {
					int len = strlen(buf);
					buttonNum = atoi(buf + len - 1);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, buttonNum]() { ref->RawButtonPressed(buttonNum); });
					}
					continue;
				}
				if (strncmp(buf, "Value", 5) == 0) {
					int len = strlen(buf);
					valueNum = atoi(buf + len - 1);
					char* s = buf + 7;
					rawVal = atof(s);
					for (int i = 0; i < refs.Num(); i++) {
						UPeelWireless* ref = refs[i];
						AsyncTask(ENamedThreads::GameThread, [ref, valueNum, rawVal]() { ref->RawValuePressed(valueNum, rawVal); });
					}
					continue;
				}
				for (int i = 0; i < refs.Num(); i++) {
					UPeelWireless* ref = refs[i];
					auto cmd = FText::FromString(ANSI_TO_TCHAR(buf));
					AsyncTask(ENamedThreads::GameThread, [ref, cmd]() { ref->OnCommand.Broadcast(cmd); });
				}
			}
		}

	}
	
	bRunThread = false;

	mSocket->Close();
	delete mSocket;


	return 0;
}

#pragma optimize("", on)