// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Delegates/Delegate.h"
#include "Components/ActorComponent.h"
#include "Async/AsyncWork.h"
#include "Async/Async.h"
#include "CineCameraComponent.h"
#include "CineCameraActor.h"
#include "MovieSceneSequencePlayer.h"
#include "LevelSequence.h"

#include "PeelWireless.generated.h"

DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FCommandEvent, FText, command);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FLensEvent, float, lens);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FApertureEvent, float, aperture);

DECLARE_DYNAMIC_MULTICAST_DELEGATE(FFilmbackEvent);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FFocusChangeEvent, float, distance);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FDebugFocusEvent, bool, enable);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FPlaySequencerEvent, bool, playing);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FTurnCameraByEvent, float, rotation);

DECLARE_DYNAMIC_MULTICAST_DELEGATE(FZeroMagicCarpetEvent);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOffsetTranslationEvent, FVector, offsetTranslation);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOffsetRotationEvent, FVector, offsetRotation);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FDampingTranslationEvent, FVector, dampingTranslation);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FDampingRotationEvent, float, dampingRotation);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FAddKeyEvent);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FPrevKeyEvent);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FNextKeyEvent);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FRawButton1Event);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FRawButton2Event);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FRawButton3Event);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FRawButton4Event);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FRawValue1Event, float, rawValue);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FRawValue2Event, float, rawValue);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FRawValue3Event, float, rawValue);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FRawValue4Event, float, rawValue);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOffsetFrameEvent, float, offsetFrame);

UDELEGATE(BlueprintCallable, CallInEditor)
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FBeginningFrameEvent);

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class PEELVCAM_API UPeelWireless : public UActorComponent
{
	GENERATED_BODY() 

public:	
	// Sets default values for this component's properties
	UPeelWireless(FObjectInitializer const& ObjectInitializer);
	virtual ~UPeelWireless() override;

protected:
	// Called when the game starts
	virtual void BeginPlay() override;
	virtual void EndPlay(const EEndPlayReason::Type) override;
	virtual void InitializeComponent() override;
	virtual void DestroyComponent(bool bPromoteChildren) override;

public:	
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

	void SetLens(float);
	void SetAperture(float);
	void SetFilmback(float, float, float);
	void SetOffsetTranslation(FVector);
	void SetOffsetRotation(FVector);
	void SetDampingTranslation(FVector);
	void SetDampingRotation(float);
	void SetFocusDistance(float);
	void SetDebugFocus(bool);
	void SetFrame(float);
	void TurnCameraBy(float);
	void ZeroMagicCarpet();
	void PrevDollyKey();
	void NextDollyKey();
	void ClearDollyKeys();
	void AddDollyTransformKey();
	void IncrementFrame();
	void DecrementFrame();
	void BeginningFrame();
	void AddLandmark(int);
	void GotoLandmark(int);
	void RawButtonPressed(int);
	void RawValuePressed(int, float);
	float currentFrame = 0.0;
	struct FLandmarkData{
		FTransform cameraWorld;
		FTransform magicCarpetWorld;
	};
	TMap<int, FLandmarkData> Landmarks;

	TWeakObjectPtr<UCineCameraComponent> FindCameraComponent();
	
	TWeakObjectPtr<AActor> FindDollyActor();
	
	TWeakObjectPtr<AActor> FindMagicCarpetActor();

	TWeakObjectPtr<ACineCameraActor> FindCineCameraActor();

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FCommandEvent OnCommand;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FLensEvent OnLens;
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FApertureEvent OnAperture;
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FFilmbackEvent OnFilmbackChange;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FFocusChangeEvent OnFocus;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FDebugFocusEvent OnDebugFocus;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FPlaySequencerEvent OnPlaySequencer;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FOffsetTranslationEvent OnOffsetTranslation;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FOffsetRotationEvent OnOffsetRotation;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FDampingTranslationEvent OnDampingTranslation;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FDampingRotationEvent OnDampingRotation;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FTurnCameraByEvent OnTurnCameraBy;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FZeroMagicCarpetEvent OnZeroMC;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawButton1Event OnButton1Event;
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawButton2Event OnButton2Event;
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawButton3Event OnButton3Event;
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawButton4Event OnButton4Event;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawValue1Event OnValue1Event;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawValue2Event OnValue2Event;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawValue3Event OnValue3Event;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FRawValue4Event OnValue4Event;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FOffsetFrameEvent OnOffsetFrame;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FAddKeyEvent OnAddKey;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, CallInEditor, Category = "PeelDev")
	FBeginningFrameEvent OnBeginningFrameEvent;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float Lens = 0;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float Aperture = 0;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float FocusDistance = 100;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float RawValue1 = 0;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float RawValue2 = 0;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float RawValue3 = 0;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float RawValue4 = 0;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	bool DebugFocus = false;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	bool IsPlaying = false;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	FVector OffsetTranslation;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	FVector OffsetRotation;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "PeelDev")
	FVector DampingTranslation;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "PeelDev")
	float DampingRotation;

	UPROPERTY(EditAnywhere, Blueprintable, Category = "PeelDev")
	float OffsetFrame = 0.0f;
	
};


